function pred_model = bicycle_pred_model_TEMP(x0, ref, u0, Ts, PointAndTangent, U_star, lpv)


% operative state
vx = x0(1);
vy = x0(2);
wz = x0(3);
s = x0(4);
ey = x0(5);
epsi = x0(6);

% operative input
% u0 = [0; 0];
delta_f = 0;
a = 0;



%% Constants
lf = 0.75;
lr = 0.75;
L = lf+lr;
W = 0.8;
m = 800;
Iz = m*(L^2 + W^2)/12;
Cf = 10000;
Cr = 10000;


    
%% Curvature
s0 = x0(4);
index = find(s0 >= PointAndTangent(:,4) & s0 < PointAndTangent(:,4)+PointAndTangent(:,5), 1);
c = PointAndTangent(index,6);


%% Operative point
theta_f = (vy + lf*wz)/vx;
theta_r = (vy - lr*wz)/vx;

% sideslip angles
alpha_f = delta_f - theta_f;
alpha_r = -theta_r;

% F_yf = 2*Cf*alpha_f;
% F_yr = 2*Cr*alpha_r;


alpha_lim = 5*pi/180;      % 5° = 0.0873 rad
if (alpha_f <= alpha_lim && alpha_f >= -alpha_lim)
    F_yf = 2*Cf*alpha_f;
else
    theta_f = alpha_lim*sign(alpha_f);
    F_yf = 2*Cf*theta_f;
end

if (alpha_r <= alpha_lim && alpha_r >= -alpha_lim)
    F_yr = 2*Cr*alpha_r;
else
    theta_r = alpha_lim*sign(alpha_r);
    F_yr = 2*Cr*theta_r;
end


% dynamics
vx_dot = a - 1/m*(F_yf*sin(delta_f)) + wz*vy;
vy_dot = 1/m*(F_yf*cos(delta_f) + F_yr) - wz*vx;
wz_dot = 1/Iz*(lf*F_yf*cos(delta_f) - lr*F_yr);
s_dot = (vx*cos(epsi) - vy*sin(epsi))/(1-c*ey);
ey_dot = vx*sin(epsi) + vy*cos(epsi);
epsi_dot = wz - (vx*cos(epsi) - vy*sin(epsi))/(1-c*ey)*c;

x_dot0 = [vx_dot; vy_dot; wz_dot; s_dot; ey_dot; epsi_dot];


%% delta_f_dot
if size(U_star, 2) < 2
    delta_f_dot0 = delta_f/Ts;
else
    delta_f_dot0 = (delta_f-U_star(1,end-1))/Ts;
end



%% ==================================================================
%  =========================== JACOBIAN =============================
%  ==================================================================

%% A11
dvx_vx = -(2*Cf*sin(delta_f)*theta_f)/(m*vx);
dvx_vy = (2*Cf*sin(delta_f))/(m*vx) + wz;
dvx_wz = (2*Cf*lf*sin(delta_f))/(m*vx) + vy;

dvy_vx = 2*(Cf*theta_f*cos(delta_f) + Cr*theta_r)/(m*vx) - wz;
dvy_vy = -2*(Cf*cos(delta_f) + Cr)/(m*vx);
dvy_wz = 2*(Cf*lf*cos(delta_f) - Cr*lr)/(m*vx) - vx;

dwz_vx = 2*(Cf*lf*theta_f*cos(delta_f) - Cr*lr*theta_r)/(Iz*vx);
dwz_vy = -2*(Cf*lf*cos(delta_f) - Cr*lr)/(Iz*vx);
dwz_wz = -2*(Cf*lf^2*cos(delta_f) + Cr*lr^2)/(Iz*vx);

A11 = [dvx_vx,  dvx_vy,  dvx_wz;
       dvy_vx,  dvy_vy,  dvy_wz;
       dwz_vx,  dwz_vy,  dwz_wz];


   
%% A22
ds_s = 0;
ds_ey = (vx*cos(epsi) - vy*sin(epsi))/((1 - c*ey).^2)*c;
ds_epsi = -(vx*sin(epsi) + vy*cos(epsi))/(1 - c*ey);

dey_s = 0;
dey_ey = 0;
dey_epsi = (vx*cos(epsi) - vy*sin(epsi));

depsi_s = 0;
depsi_ey = -(vx*cos(epsi) - vy*sin(epsi))/((1 - c*ey).^2)*c.^2;
depsi_epsi = (vx*sin(epsi) + vy*cos(epsi))/(1 - c*ey)*c;

A22 = [ds_s,     ds_ey,     ds_epsi;
       dey_s,    dey_ey,    dey_epsi;
       depsi_s,  depsi_ey,  depsi_epsi];
   
   
%% A12
dvx_s = 0;
dvx_ey = 0;
dvx_epsi = 0;

dvy_s = 0;
dvy_ey = 0;
dvy_epsi = 0;

dwz_s = 0;
dwz_ey = 0;
dwz_epsi = 0;

A12 = [dvx_s,  dvx_ey,  dvx_epsi;
       dvy_s,  dvy_ey,  dvy_epsi;
       dwz_s,  dwz_ey,  dwz_epsi];


%% A21
ds_vx = cos(epsi)/(1 - c*ey);
ds_vy = -sin(epsi)/(1 - c*ey);
ds_wz = 0;

dey_vx = sin(epsi);
dey_vy = cos(epsi);
dey_wz = 0;

depsi_vx = -cos(epsi)/(1 - c*ey)*c;
depsi_vy = sin(epsi)/(1 - c*ey)*c;
depsi_wz = 1;

A21 = [ds_vx,     ds_vy,     ds_wz;
       dey_vx,    dey_vy,    dey_wz;
       depsi_vx,  depsi_vy,  depsi_wz];
   
   
%% B1
dvx_delta_f = -2*Cf/m*(sin(delta_f) + (delta_f - theta_f)*cos(delta_f));
% dvx_delta_f = -2*Cf/m*(sin(delta_f));
dvx_a = 1;

dvy_delta_f = 2*Cf/m*(cos(delta_f) - (delta_f - theta_f)*sin(delta_f));
% dvy_delta_f2 = 2*Cf/m*(cos(delta_f));
dvy_a = 0;

dwz_delta_f = 2*Cf*lf/m*(cos(delta_f) - (delta_f - theta_f)*sin(delta_f));
% dwz_delta_f = 2*Cf*lf/m*(cos(delta_f));
dwz_a = 0;


B1 = [dvx_delta_f,  dvx_a;
      dvy_delta_f,  dvy_a;
      dwz_delta_f,  dwz_a];
  
  
%% B2
ds_delta_f = 0;
ds_a = 0;

dey_delta_f = 0;
dey_a = 0;

depsi_delta_f = 0;
depsi_a = 0;

B2 = [ds_delta_f,    ds_a;
      dey_delta_f,   dey_a;
      depsi_delta_f, depsi_a];

  
%% QP matrices
A = [A11 A12
      A21 A22];
A(isnan(A)) = 0;
A(A==+inf) = 0;
A(A==-inf) = 0;

B = [B1;
      B2];
B(isnan(B)) = 0;
B(B==+inf) = 0;
B(B==-inf) = 0;

if lpv == false
    K = -B*[0; 0];
elseif lpv == true
    K = x_dot0 - A*x0 - B*u0;
end


%% Discretization
Ad = eye(size(A)) + A*Ts;
Bd = B*Ts;
Kd = K*Ts;



%% Structure
pred_model.Ad = Ad;
pred_model.Bd = Bd;
pred_model.Kd = Kd;




end