function x_plus = bicycle_lin_model_aug_states(x0, ref, u0, Ts, PointAndTangent, U_star, lpv)

lpv = true;

% operative state
vx = x0(1);
vy = x0(2);
wz = x0(3);
s = x0(4);
ey = x0(5);
epsi = x0(6);
delta_f = x0(7);

% operative input
delta_f_dot = u0(1);
a = u0(2);



%% Constants
lf = 0.75;
lr = 0.75;
L = lf+lr;
W = 0.8;
m = 800;
Iz = m*(L^2 + W^2)/12;
Cf = 10000;
Cr = 10000;


    
%% Curvature
s0 = x0(4);
index = find(s0 >= PointAndTangent(:,4) & s0 < PointAndTangent(:,4)+PointAndTangent(:,5), 1);
c = PointAndTangent(index,6);



%% Operative point

% sideslip angles
% alpha_f = delta_f - atan2((vy + lf*wz),vx);
% alpha_r = -atan2((vy - lr*wz),vx);
alpha_f = delta_f - (vy + lf*wz)/vx;
alpha_r = -(vy - lr*wz)/vx;

alpha_lim = 5*pi/180;      % 5° = 0.0873 rad

if (alpha_f <= alpha_lim && alpha_f >= -alpha_lim)
    theta_f = alpha_f;
    F_yf = 2*Cf*alpha_f;
else
    theta_f = 5*pi/180;
    F_yf = 2*Cf*alpha_lim*sign(alpha_f);
end

if (alpha_r <= alpha_lim && alpha_r >= -alpha_lim)
    F_yr = 2*Cr*alpha_r;
else
    F_yr = 2*Cr*alpha_lim*sign(alpha_r);
end

% dynamics
vx_dot0 = a - 1/m*(F_yf*sin(delta_f)) + wz*vy;
vy_dot0 = 1/m*(F_yf*cos(delta_f) + F_yr) - wz*vx;
wz_dot0 = 1/Iz*(lf*F_yf*cos(delta_f) - lr*F_yr);
s_dot0 = (vx*cos(epsi) - vy*sin(epsi))/(1-c*ey);
ey_dot0 = vx*sin(epsi) + vy*cos(epsi);
epsi_dot0 = wz - (vx*cos(epsi) - vy*sin(epsi))/(1-c*ey)*c;
delta_f_dot0 = u0(1);

x_dot0 = [vx_dot0; vy_dot0; wz_dot0; s_dot0; ey_dot0; epsi_dot0; delta_f_dot0];


%% delta_f_dot
% if size(U_star, 2) < 2
%     delta_f_dot0 = delta_f/Ts;
% else
%     delta_f_dot0 = (delta_f-U_star(1,end-1))/Ts;
% end


%% ==================================================================
%  =========================== JACOBIAN =============================
%  ==================================================================


%% A11
dvx_vx = -(2*Cf*sin(delta_f)*theta_f)/(m*vx);
dvx_vy = (2*Cf*sin(delta_f))/(m*vx) + wz;
dvx_wz = (2*Cf*lf*sin(delta_f))/(m*vx) + vy;

dvy_vx = 2*((Cf*(vy + lf*wz))*cos(delta_f)/vx + (Cr*(vy - lr*wz)/vx))/(m*vx) - wz;
% dvy_vx = 2*((Cf*theta_f)*cos(delta_f) - (Cr*-(vy - lr*wz)/vx))/(m*vx) - wz;
dvy_vy = -2*(Cf*cos(delta_f) + Cr)/(m*vx);
dvy_wz = 2*(Cf*lf*cos(delta_f) - Cr*lr)/(m*vx) - vx;

dwz_vx = 2*(Cf*lf*(vy + lf*wz)*cos(delta_f)/vx - Cr*lr*(vy - lr*wz)/vx)/(Iz*vx);
% dwz_vx = 2*(Cf*lf*theta_f*cos(delta_f) + Cr*lr*-(vy - lr*wz)/vx)/(Iz*vx);
dwz_vy = -2*(Cf*lf*cos(delta_f) - Cr*lr)/(Iz*vx);
dwz_wz = -2*(Cf*cos(delta_f)*lf^2 + Cr*lr^2)/(Iz*vx);

A11 = [dvx_vx,  dvx_vy,  dvx_wz;
       dvy_vx,  dvy_vy,  dvy_wz;
       dwz_vx,  dwz_vy,  dwz_wz];


%% A22
ds_s = 0;
ds_ey = c*(vx*cos(epsi) - vy*sin(epsi))/(1 - c*ey).^2;
ds_epsi = -(vx*sin(epsi) + vy*cos(epsi))/(1 - c*ey) * epsi_dot0;
% ds_epsi = -(vx*sin(epsi) + vy*cos(epsi))/(1 - c*ey);
ds_delta = 0;

dey_s = 0;
dey_ey = 0;
dey_epsi = (vx*cos(epsi) - vy*sin(epsi)) * epsi_dot0;
% dey_epsi = (vx*cos(epsi) - vy*sin(epsi));
dey_delta = 0;

depsi_s = 0;
depsi_ey = -c.^2*(vx*cos(epsi) - vy*sin(epsi))/(1 - c*ey).^2;
depsi_epsi = c*(vx*sin(epsi) + vy*cos(epsi))/(1 - c*ey) * epsi_dot0;
% depsi_epsi = c*(vx*sin(epsi) + vy*cos(epsi))/(1 - c*ey);
depsi_delta = 0;

ddelta_s = 0;
ddelta_ey = 0;
ddelta_epsi = 0;
ddelta_delta = 0;

A22 = [ds_s,      ds_ey,       ds_epsi,       ds_delta;
       dey_s,     dey_ey,      dey_epsi       dey_delta;
       depsi_s,   depsi_ey,    depsi_epsi     depsi_delta
       ddelta_s,  ddelta_ey,   ddelta_epsi,   ddelta_delta];
   


%% A12
dvx_s = 0;
dvx_ey = 0;
dvx_epsi = 0;
% dvx_delta = -2*Cf/m*(sin(delta_f) + (delta_f - theta_f)*cos(delta_f)*delta_f_dot);
dvx_delta = -2*Cf/m*(sin(delta_f));

dvy_s = 0;
dvy_ey = 0;
dvy_epsi = 0;
% dvy_delta = 2*Cf/m*(cos(delta_f) - (delta_f - theta_f)*sin(delta_f)*delta_f_dot);
dvy_delta = 2*Cf/m*(cos(delta_f));

dwz_s = 0;
dwz_ey = 0;
dwz_epsi = 0;
% dwz_delta = 2*Cf*lf/m*(cos(delta_f) - (alpha_f)*sin(delta_f)*delta_f_dot);
dwz_delta = 2*Cf*lf/m*(cos(delta_f));

A12 = [dvx_s,  dvx_ey,  dvx_epsi,  dvx_delta;
       dvy_s,  dvy_ey,  dvy_epsi,  dvy_delta;
       dwz_s,  dwz_ey,  dwz_epsi,  dwz_delta];



%% A21
ds_vx = -cos(epsi)/(c*ey - 1);
ds_vy = sin(epsi)/(c*ey - 1);
ds_wz = 0;

dey_vx = sin(epsi);
dey_vy = cos(epsi);
dey_wz = 0;

depsi_vx = -c*cos(epsi)/(1 - c*ey);
depsi_vy = c*sin(epsi)/(1 - c*ey);
depsi_wz = 1;

ddelta_vx = 0;
ddelta_vy = 0;
ddelta_wz = 0;

A21 = [ds_vx,      ds_vy,      ds_wz;
       dey_vx,     dey_vy,     dey_wz;
       depsi_vx,   depsi_vy,   depsi_wz
       ddelta_vx,  ddelta_vy,  ddelta_wz];
   

%% B1
dvx_ddelta = 0;
dvx_a = 1;

dvy_ddelta = 0;
dvy_a = 0;

dwz_ddelta = 0;
dwz_a = 0;

B1 = [dvx_ddelta,  dvx_a;
      dvy_ddelta,  dvy_a;
      dwz_ddelta,  dwz_a];
  
  
%% B2
ds_ddelta = 0;
ds_a = 0;

dey_ddelta = 0;
dey_a = 0;

depsi_ddelta = 0;
depsi_a = 0;

ddelta_ddelta = 1;
ddelta_a = 0;

B2 = [ds_ddelta,      ds_a;
      dey_ddelta,     dey_a;
      depsi_ddelta,   depsi_a
      ddelta_ddelta,  ddelta_a];
  
  
  
%% QP matrices
A = [A11 A12
      A21 A22];
A(isnan(A)) = 0;
A(A==+inf) = 0;
A(A==-inf) = 0;

B = [B1;
      B2];
B(isnan(B)) = 0;
B(B==+inf) = 0;
B(B==-inf) = 0;

if lpv == false
    K = -B*u0;
elseif lpv == true
    K = x_dot0 - A*x0 - B*u0;
end


%% Discretization
Ad = eye(size(A)) + A*Ts;
Bd = B*Ts;
Kd = K*Ts;



%% Model
u = U_star(1:2, end);
x_plus = Ad*x0 + Bd*u + Kd;


end