%% MAIN
% close all
clear all
clc

addpath('../00 - Functions/mpc','-end')
addpath('../00 - Functions/plant_modelling','-end')
addpath('../00 - Functions/plot','-end')
addpath('../00 - Functions/utils','-end')



%% =================================================================
%  ============================ Plant ============================
%  =================================================================

% UGV = [vx, vy, wz, X, Y, psi];
x0 = [1.5; 0; 0; 0; 0; 90*pi/180];
% x0 = [1.5; 0; 0; 0; 0; 140*pi/180];
% x0 = [1.5; 0; 0; 0.8; 2; 90*pi/180];
% x0 = [1.5; 0; 0; -0.8; 1; 60*pi/180];

color = 'b';

T = 0.001;

figure(1); hold on; axis equal
title('Trajectory');
% p1(1) = plot(x0(4), x0(5), 'or');
% arrow(x0(4), x0(5), x0(6), 'r');
% set(get(get(p1(1), 'Annotation'), 'LegendInformation'), 'IconDisplayStyle', 'off');

% vehicle data
sys_model.lf = 0.75;
sys_model.lr = 0.75;
sys_model.L = sys_model.lf+sys_model.lr;
sys_model.W = 0.8;
sys_model.m = 800;
sys_model.Iz = sys_model.m*(sys_model.L^2 + sys_model.W^2)/12;
sys_model.Cf = 10000;
sys_model.Cr = 10000;



%% =================================================================
%  ============================ Track ==============================
%  =================================================================

track_n = 3;
W_track = 1;
[PointAndTangent, x0_track] = track_fun(track_n, W_track);



%% ==================================================================
%  ======================= MPC initialization =======================
%  ==================================================================

% initial condition
X_plant = x0;
[s0, ey0, epsi0] = global2local_fun(x0(4:6), x0_track, PointAndTangent, W_track);
X_MPC = [x0(1); x0(2); x0(3); s0; ey0; epsi0];
% X_MPC = [0.1; 0; 0; 0.1; 0; 0];


% target
target = [PointAndTangent(end,1); PointAndTangent(end,2); PointAndTangent(end,3)];
[s_target, ey_target, epsi_target] = global2local_fun(target, x0(4:6), PointAndTangent, W_track);
% ref = [1.5; 0; 0; s_target-s0; ey_target; epsi_target];
ref = [0; 0; 0; 0; 0; 0];


% state constraints [vx, vy, wz, s, ey, epsi]
x_c = [-0.2,           +0.2
       -0.5,           +0.5
       -50*pi/180,  +50*pi/180
       -0.5,           +0.5
       -W_track,          +W_track
       -50*pi/180,  +50*pi/180];

% input constraints [delta_f, a]
u_c = [-30*pi/180,  +30*pi/180
       -0.5,          +0.1];

% input rate constraints [delta_f, a]
du_c = [-2*pi/180,     +2*pi/180
        -0.05,           +0.05];



%% ==================================================================
%  ============================== MPC ===============================
%  ==================================================================

T_nav = 0.02;
method = 0;
lpv = false;

plant = 1;      % 1 - nonlin. cont. plant, 2 - nonlin. disc. plant, 3 - lin. disc. plant
if plant==1
    model_color = 'r';
    model_name = 'nonlin. cont. plant';
elseif plant==2
    model_color = 'b';
    model_name = 'nonlin. disc. plant';
elseif plant==3
    model_color = 'g';
    model_name = 'lin. disc. plant';
end



% MPC tuning
Np = 20;
Nc = Np;
Q = diag([1 1 1 1 5 1]);
R = diag([1 1]);
S = diag([1 1]);



k = 1;
flag = 0;

U_old = zeros(Nc*2,1);
U_MPC = [0; 0];
J = 0;

t_sim = 100;
while (flag == 0)
    
    clc
    t = T_nav*k;
    
    %% Print info
    fprintf('Sample: k = %d, time: t = %.2f\n', [k, t]);
    fprintf('Np = %d, Nc = %d \n', [Np, Nc]);
    fprintf('Velocity =         [vx = %.2fm/s,  vy = %.2fm/s,   wz =   %.1f°/s] \n', [X_MPC(1,k), X_MPC(2,k), X_MPC(3,k)*180/pi]);
    fprintf('Curvilinear pose = [s =  %.2fm,    ey = %.2fm,     epsi = %.1f°] \n',   [X_MPC(4,k), X_MPC(5,k), X_MPC(6,k)*180/pi]);
    fprintf('Cost = [J =  %.2f] \n',   [J]);


    %% MPC
    U_star = adaptive_mpc(X_MPC(:,k), X_MPC(:,1), ref, U_old, U_MPC, T_nav, Np, Nc, Q, R, S, x_c, u_c, du_c, PointAndTangent, method, lpv);
%     U_star = [10*pi/180; 0];
    if (isempty(U_star) | isnan(U_star))
        u_star = [];
        fprintf('Problem infeasible at k = %d \n', k);
        t_fin = T_nav*(k-1);
        break;
    end
    U_old = U_star;
    U_MPC(:,k) = U_star(1:2);
    fprintf('Input =            [delta_f = %.1f°,  a = %.2fm/s^2]\n', [U_MPC(1,k)*180/pi, U_MPC(2,k)]);


    %% Plant
    
    if plant==1                             % nonlinear continuos plant
        x = X_plant(:,k);
        u = U_MPC(:,k);
        
        [t_x, x_plus] = ode23(@(t,x)bicycle_dynamics_continuous(x, u, sys_model), linspace(0, T_nav, T_nav/T), x);
        x = x_plus(end,:);
        
        X_plant(:,k+1) = x;
        
        % Conversion from global to curvilinear coordinates
        [s, ey, epsi, CompletedFlag] = global2local_fun(X_plant(4:6,k+1), x0_track, PointAndTangent, W_track);
        X_MPC(:,k+1) = [X_plant(1,k+1); X_plant(2,k+1); X_plant(3,k+1); s; ey; epsi];
        
    elseif plant==2                         % nonlinear discrete plant
        x = X_plant(:,k);
        u = U_MPC(:,k);
        
        for i=1:T_nav/T
            [x_plus, alpha] = bicycle_dynamics_discrete(x, u, sys_model, T);
            x = x_plus;
        end
        
        X_plant(:,k+1) = x;
        
        % Conversion from global to curvilinear coordinates
        [s, ey, epsi, CompletedFlag] = global2local_fun(X_plant(4:6,k+1), x0_track, PointAndTangent, W_track);
        X_MPC(:,k+1) = [X_plant(1,k+1); X_plant(2,k+1); X_plant(3,k+1); s; ey; epsi];
        
    elseif plant==3                         % linear discrete plant
        x = X_MPC(:,k);
        u = U_MPC(:,k);
        for i=1:T_nav/T
            x_plus = bicycle_lin_model(x, ref, u, T, PointAndTangent, U_star, lpv);
            x = x_plus;
        end
        
        X_MPC(:,k+1) = x';
        [x_plant, y_plant] = local2global_fun(X_MPC(4:6,k+1), x0(4:6), PointAndTangent);
        X_plant(:,k+1) = [X_MPC(1:3,k+1); x_plant; y_plant; 0];
        CompletedFlag = 1;
    end
    

    %% Update sample
    k = k+1;
    u_old = U_MPC(:,k-1);



    
    %% Exit condition
    if (abs(X_MPC(4,k) - s_target) <= 10e-2)
        fprintf('Terget reached \n');
        flag = 1;
        t_fin = T_nav*(k-1);
    elseif (t == t_sim)
        fprintf('Simulation time reached \n');
        flag = 1;
        t_fin = T_nav*(k);
    elseif CompletedFlag == 0
        flag = 1;
        t_fin = T_nav*(k);
        break;
    end
        
end



%% ==================================================================
%  =========================== Plot =================================
%  ==================================================================

t_vec = linspace(0, t_fin, (t_fin+T_nav)/T_nav);
% leg_name = 'trajectory';
% leg_name = 'x0 = [' + string(x0(1)) + ', ' + string(x0(2)) + ', ' + string(x0(3)*180/pi) + ', ' + string(x0(4))  + ', ' + string(x0(5)) + ', ' + string(x0(6)*180/pi)+ ']';
leg_name = 'Q = [' + string(Q(1,1)) + ', ' + string(Q(2,2)) + ', ' + string(Q(3,3)) + ', ' + string(Q(4,4))  + ', ' + string(Q(5,5)) + ', ' + string(Q(6,6)) + ']; ' + ...
           'R = [' + string(R(1,1)) + ', ' + string(R(2,2)) + ']; ' + ...
           'S = [' + string(S(1,1)) + ', ' + string(S(2,2)) + ']';
% leg_name = 'v_x = ' + string(x0(1));
% leg_name = 'N_p = ' + string(Np);

figure(1); hold on; axis equal;
title('Trajectory');
xlabel('x [m]');
ylabel('y [m]');

% p1 = plot(X_plant(4,:), X_plant(5,:), 'Color', color, 'LineStyle', '-', 'LineWidth', 1);
p1 = plot(X_plant(4,:), X_plant(5,:), 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', leg_name);

% arrow(X_plant(4,end), X_plant(5,end), X_plant(6,end), 'b');



%% UGV position
% figure(2); hold on; grid on;
% title('Position');
% m = 3; n = 1;
% 
% subplot(m, n, 1); hold on; grid on; 
% title('x');
% plot(t_vec, X_plant(4,1:end), 'k', 'DisplayName', 'x');
% 
% subplot(m, n, 2); hold on; grid on; 
% title('y');
% plot(t_vec, X_plant(5,1:end), 'k', 'DisplayName', 'y');
% 
% subplot(m, n, 3); hold on; grid on; 
% title('psi');
% plot(t_vec, X_plant(6,1:end), 'k', 'DisplayName', 'psi');


%% UGV curvilinear
figure(3); 
title('Curvilinear states');
m = 3; n = 1;

subplot(m, n, 1); hold on; grid on;
title('s');
xlabel('t [s]');
ylabel('y [m]');
plot(t_vec, X_MPC(4,1:end), 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', 's');
% yline(x_c(4,1), 'r--'); yline(x_c(4,2), 'r--');

subplot(m, n, 2); hold on; grid on;
title('e_y');
xlabel('t [s]');
ylabel('y [m]');
% ylim([x_c(5,1)-x_c(5,1)*2/100, x_c(5,2)+x_c(5,2)*2/100]);
plot(t_vec, X_MPC(5,1:end), 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', 'e_y');
% yline(x_c(5,1), 'r--'); yline(x_c(5,2), 'r--');

subplot(m, n, 3); hold on; grid on;
title('e_{\Psi}');
xlabel('t [s]');
ylabel('deg [°]');
% ylim([x_c(6,1)*180/pi+x_c(6,1)*180/pi*10/100, x_c(6,2)*180/pi+x_c(6,2)*180/pi*10/100]);
plot(t_vec, X_MPC(6,1:end)*180/pi, 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', 'e_{\Psi}');
% yline(x_c(6,1)*180/pi, 'r--'); yline(x_c(6,2)*180/pi, 'r--');


%% UGV velocity
figure(4); hold on; grid on;
title('Velocity');
m = 3; n = 1;

subplot(m, n, 1); hold on; grid on;
title('v_x');
xlabel('t [s]');
ylabel('v_x [m/s]');
plot(t_vec, X_MPC(1,1:end), 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', 'v_x');

subplot(m, n, 2); hold on; grid on;
title('v_y');
xlabel('t [s]');
ylabel('v_y [m/s]');
plot(t_vec, X_MPC(2,1:end), 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', 'v_y');

subplot(m, n, 3); hold on; grid on;
title('\omega_z');
xlabel('t [s]');
ylabel('\omega_z [°/s]');
plot(t_vec, X_MPC(3,1:end)*180/pi, 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', '\omega_z');


%% UGV input
figure(5); hold on; grid on;
title('Input');
m = 2; n = 1;

subplot(m, n, 1); hold on; grid on;
title('\delta_f');
xlabel('t [s]');
ylabel('\delta_f [°]');
% ylim([u_c(1,1)*180/pi+u_c(1,1)*180/pi*10/100, u_c(1,2)*180/pi+u_c(1,2)*180/pi*10/100]);
plot(t_vec(1:end-1), U_MPC(1,1:end)*180/pi, 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', '\delta_f');

% plot(t_vec(1:end-1), alpha_f(1:end)*180/pi, 'b', 'DisplayName', 'alpha_f');
yline(u_c(1,1)*180/pi, 'r--'); yline(u_c(1,2)*180/pi, 'r--');

subplot(m, n, 2); hold on; grid on;
title('a');
xlabel('t [s]');
ylabel('a [m/s^2]');
% ylim([u_c(2,1)+u_c(2,1)*10/100 u_c(2,2)+u_c(2,2)*10/100]);
plot(t_vec(1:end-1), U_MPC(2,1:end), 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', 'a');
yline(u_c(2,1), 'r--'); yline(u_c(2,2), 'r--');



%% UGV steering velocity
for i=2:length(U_MPC)
    delta_f_dot(i-1) = (U_MPC(1,i)-U_MPC(1,i-1))/T_nav;
end

figure(6); hold on; grid on;
title('Steering velocity');
xlabel('t [s]');
ylabel('d\delta_{f} [°/s]');
plot(t_vec(1:end-2), delta_f_dot(1,1:end)*180/pi, 'Color', color, 'LineStyle', '-', 'LineWidth', 1, 'DisplayName', 'd\delta_{f}');


return

%% Save plots
prompt = 'Do you want to save the plots? y/n [n]: ';
str = input(prompt,'s');
if str == 'y'
    
    save('data/trajectory.mat', 'X_MPC', 'X_plant', 'U_MPC', 't_fin', 'T_nav')
    
    % mpc params
    mpc.Np = Np;
    mpc.Nc = Nc;
    mpc.Q = Q;
    mpc.R = R;
    save data/mpc_data.mat sys_model Np Q R S
    
    % trajectory plot
    filename = sprintf('data/trajectory.png');
    exportgraphics(figure(1), filename);
    
    % trajectory plot
    filename = sprintf('data/velocities.png');
    exportgraphics(figure(4), filename);
    
    % trajectory plot
    filename = sprintf('data/curvliniear_coordinates.png');
    exportgraphics(figure(3), filename);
    
    % trajectory plot
    filename = sprintf('data/input.png');
    exportgraphics(figure(5), filename);
    
end

close all


